/**
 * Set of functions for graph rendering 
 * Author: Marcio Fuckner
 *
 * Revision history:
 * 05/11/2013 - MFK - Bug correction allowing several divs exhibition.
 * 03/10/2014 - MFK - Ontocodesign features (inverse relations, relations by reference)
 */
 
var labelType, useGradients, nativeTextSupport, animate;

(function() {
  var ua = navigator.userAgent,
      iStuff = ua.match(/iPhone/i) || ua.match(/iPad/i),
      typeOfCanvas = typeof HTMLCanvasElement,
      nativeCanvasSupport = (typeOfCanvas == 'object' || typeOfCanvas == 'function'),
      textSupport = nativeCanvasSupport 
        && (typeof document.createElement('canvas').getContext('2d').fillText == 'function');
  //I'm setting this based on the fact that ExCanvas provides text support for IE
  //and that as of today iPhone/iPad current text support is lame
  labelType = (!nativeCanvasSupport || (textSupport && !iStuff))? 'Native' : 'HTML';
  nativeTextSupport = labelType == 'Native';
  useGradients = nativeCanvasSupport;
  animate = !(iStuff || !nativeCanvasSupport);
})(); 

// global variables 
var colors = [], // array of available node colors
    shapes = [], // array of available shape types
    conceptStyles = {},  // array of styles
	  colorPos, // last color used
	  shapePos, // last shape used
	  sequence = 0, // last object sequence
    currentNode = null;

/**
 * Render a graph in the specified div
 * Note: API dependent function (Infovis Toolkit)
 * Parameters:
 *   obj - the json representation of the graph
 *   labelFunctions - the set of label functions
 *   divName - the target div
 */ 

function renderGraph(objs,divName){

  initColors();
  initShapes();

  conceptStyles = {};	
  
  sequence ++;
  
  $jit.RGraph.Plot.EdgeTypes.implement({
    'labeled': {
    'render': function(adj, canvas) {
                //this.edgeTypes.line.render.call(this, adj, canvas);
                this.edgeTypes.arrow.render.call(this, adj, canvas);
                var data = adj.data;
                if(data.relation && data.relation != "result") {
                  var ctx = canvas.getCtx();
                  var posFr = adj.nodeFrom.pos.getc(true);
                  var posTo = adj.nodeTo.pos.getc(true);
                  if(data.disp)
                    ctx.fillText(data.relation.relation, 
                      (posFr.x + posTo.x)/2 , (posFr.y + posTo.y)/2);
                }
              }
  }});
  
  // label function
  var createLabelFunction = function(domElement, node){ 
      var label = node.data["marshaller::summary"];
      var uid = node.data["marshaller::moss-id"];
      var concept = node.data["marshaller::type"];
      domElement.innerHTML = "<div class='lbl'><center>" + label + "&nbsp;<br>" +
          "<span class='editstyle'><a href='javascript:openEditor(" +
            "\"" + concept + "\"," + 
            "\"" + uid + "\"" +
            ")'>[edit]</a></span></center></div>";      
      domElement.onclick = function () {
  	    rgraph.onClick(node.id, { 
          hideLabels: false,
          onComplete: function() {
            // none
          }
        });
      };
  };
  
  //RGraph constructor.
  var rgraph = new $jit.RGraph({
    'injectInto': divName,
    // canvas circles
    'background': {
      'CanvasStyles': {
        'strokeStyle': '#c0c0c0',
        'shadowBlur': 100,
        'shadowColor': '#ffffff',
      }
    },
    Navigation: {  
      enable: true,  
      panning: true,
      zooming: 10 //zoom speed. higher is more sensible  
    },
    Node: {
      'overridable': true,
      'dim': 25,
      'CanvasStyles': {  
        shadowColor: '#ccc',  
        shadowBlur: 10,
      }
    },
    Edge: {
      'overridable': true,
      'color': '#cccc00',
      'type': 'line',
      'alpha':0.8,
      'dim':28,
      'lineWidth':0.5,
      'CanvasStyles': {  
        shadowColor: '#ccc',  
        shadowBlur: 10,
      }
    },    
    Events: {
      enable: true,
      enableForEdges: true,
      type: 'Native',
      onMouseEnter: function (node, eventInfo,e) {
        if(!node) 
          return;
          
        if(!node.Node) {
          var from = node.nodeFrom.data.label[0];
          var to = node.nodeTo.data.label[0];
          var role = node.data.relation.relation;
          $('#edgetip').empty();
          $('#edgetip').append(role + "&nbsp;(&nbsp;" + from + "&nbsp;)&nbsp;=&nbsp;" + to);
          $('#edgetip').css('position', 'fixed');
          $('#edgetip').css('top', e.pageY);
          $('#edgetip').css('left', e.pageX);
          $('#edgetip').css('width', 'auto');
          $('#edgetip').css('visibility', 'visible');
        }           
      },
      onMouseLeave: function(node, eventInfo,e) {
        $('#edgetip').css('visibility', 'hidden');
      },
      // click handler to node
      onClick: function(node) {
        if(currentNode) {
           currentNode.eachAdjacency(function(adj) {  
              adj.data.disp = false;
              adj.data.$alpha = 0.8;
              adj.data.$lineWidth = 0.5;
          });
        }
        if(node) {
          node.eachAdjacency(function(adj) {  
            adj.data.disp = true;
            adj.data.$alpha = 1;
            adj.data.$lineWidth = 1.5;
            });
          currentNode = node;
        };
                
      },
      onDragMove: function(node, eventInfo, e){  
        var pos = eventInfo.getPos();  
        node.pos.setc(pos.x, pos.y);  
        rgraph.plot();  
      },  
      onDragEnd: function(node, eventInfo, e){  
        rgraph.compute('end');  
        rgraph.fx.animate( {  
          modes: [  
            'linear'  
          ],  
          duration: 700,  
          transition: $jit.Trans.Elastic.easeOut  
        });  
      }        
    },
    Tips: {
      enable: true,
      onShow: function(tip, node) {
        //count connections
        var count = 0;
        if(node) {
          //display node info in tooltip
          tip.innerHTML = "<div class=\"tip-title\">" + node.data["marshaller::summary"] + "</div>"
            + "<div class=\"tip-text\">" + showDetails(node) + "</div>";            
        } else {
          alert("not a node");
        }
      }
    },
		
    // Set polar interpolation.
    // Default's linear.
    interpolation: 'linear',
	
    // Change the transition effect from linear
    // to elastic.
    transition: $jit.Trans.Elastic.easeOut,
    
    //Change other animation parameters.
    duration:2000,
    fps: 30,
    //Change father-child distance.
    levelDistance: 170,

	// This method is called right before plotting
    // an edge. This method is useful to change edge styles
    // individually.
    onBeforePlotNode: function(node){
      // resize labels
      var x = rgraph.canvas.scaleOffsetX,
          y = rgraph.canvas.scaleOffsetY;
          
      $(".node").css("-moz-transform", "scale(" + x +"," +  y + ")");
      $(".node").css("-webkit-transform", "scale(" + x +"," + y + ")");
      $(".node").css("-ms-transform", "scale(" + x +"," +  y + ")");
      $(".node").css("-o-transform", "scale(" + x +"," +  y + ")");
    },
    onBeforeCompute: function(node){
      // none
    },
    
    // Add node click handler and some styles.
    // This method is called only once for each node/label is created.
    onCreateLabel: createLabelFunction,
    
	// This method is called when rendering/moving a label.
    // This is method is useful to make some last minute changes
    // to node labels like adding some position offset.
    onPlaceLabel: function(domElement, node) {
		
      var style = domElement.style;
          style.display = '';
          style.cursor = 'pointer';

      var left = parseInt(style.left);
      var w = domElement.offsetWidth;
      style.left = (left - w / 2) + 'px';
    }
  });
  
  objs = filterConcepts(objs);
  
  // find direct object
  var directObjectId = findDirectObjectId(objs);
  
  if(directObjectId == null) {
    alert("Direct object not found");
    return;
  }
  
  // create id/object table
  var objTable = createObjectTable(objs);
    	
  // create root and related nodes	
  var root = createNode(directObjectId,objTable,rgraph);
  
  rgraph.root = root.id;
  
  //compute positions and plot
  rgraph.refresh();		
  
  return rgraph;
  
}

function findDirectObjectId(objs) {
  for(var obj in objs) {
    if(objs[obj]["marshaller::condition"] == "DIRECT") {
      return objs[obj]["marshaller::uid"];
    }
    return null;
  }
}

function createObjectTable(objs) {
	var objectTable = {};
	for(var obj in objs) {
		objectTable[objs[obj]["marshaller::uid"]] = objs[obj];
	}
	return objectTable;
}

function filterConcepts(dirtyList) {
	for(var i=dirtyList.length-1;i>=0;i--) {
		var type = dirtyList[i]["marshaller::type"];
		if(type != "CSCWD Concept") {
			dirtyList.splice(i,1);
		}
	}
	return dirtyList;
}

/**
 * Create node
 * Note: API independent function
 * Parameters:
 *   obj: source object
 *   nodes: set of created nodes
 */
function createNode(objId,objectTable,rgraph,expanded) {
	
	// if expanded list is null then create an empty one
	if(expanded == null) {
		expanded = {};
	}
	// prepare node
  var node = {};
  node.id = objId;
  node.data = {};  
	// already in the expanded list
	if(expanded[objId] != null) {
		return node;
	}	
  expanded[node.id] = node;
  
	// get related object
	var obj = objectTable[objId];
    
  // prepare visual information for node
  var nodeStyle = getStyle(obj["marshaller::type"]);
  node.data.$color = nodeStyle.$color;
	node.data.$type = nodeStyle.$type;
  node.name = obj["marshaller::summary"];
	
	var prop;
  var children = [];
	for(prop in obj) {
		if(typeof obj[prop] == 'string') {
			node.data[prop] = obj[prop];
		} else if(obj[prop] instanceof Array) {
			var arrStr = [];
			var aux = obj[prop];
      
			for(var i=0;i<aux.length;i++) {
				if(typeof aux[i] == 'string') {
          if(prop != "marshaller::uid" && prop != "marshaller::moss-id" && (aux[i].substring(0,1) == "&" || aux[i].substring(0,1) == "$" ) ) {
            if(objectTable[aux[i]] != null) {
              children.push({childId:aux[i],"relation":prop});
              arrStr.push(aux[i]);
            }
          } else {          
          	arrStr.push(aux[i]);
          } 
				} else {
					alert("invalid state!");
					return;
				}
			}
			if(arrStr.length != 0)
				node.data[prop] = arrStr;
 
		} else {
			alert("Invalid state");
			return;
		}
	}
  // add node to graph
  rgraph.graph.addNode(node);
  // add adjacencies to graph
  for(var i=0;i<children.length;i++) {
      var child = createNode(children[i].childId,objectTable,rgraph,expanded);      
      // user-friendly inverse relationship exhibition
      if(children[i].relation.indexOf("inverse::") != -1) {
        children[i].relation = children[i].relation.substring(children[i].relation.indexOf("inverse::")+9);
        var relation = {"parent" : child, "child" : node, "relation" : children[i].relation};
      } else {
        var relation = {"parent" : node, "child" : child, "relation" : children[i].relation};
      }			
      rgraph.graph.addAdjacence(node,
										child,
										{"relation":relation,
										 "labelid" : node.id + "_" + child.id});
  }  
	return node;
}

/**
 * Show node details 
 * Note: API dependent function (Infovis Toolkit)
 * Parameters:
 *   node: the node object
 *   pos: node position
 */
function showDetails(node) {
  
  html = "<ul>";
  for(var elem in node.data) {
    if(elem.substring(0,1) == "$")
      continue;
    if(elem.indexOf("marshaller::") != -1)
      continue;
    html += "<li><strong>" + elem + "</strong> = " + node.data[elem] + "</li>"
  }
  html += "</ul>";
  return html;
}

/**
 * Initialize the set of colors
 * Note: API independent function
 */
function initColors() {
    colors = [];
    colors.push("#00FF00");
    colors.push("#00FFFF");
    colors.push("#FF00FF");
    colors.push("#FFFF00");
    colors.push("#C0C0C0");
    colorPos = 0;
}

/**
 * get next available color
 */
function getNextColor() {
    return colors[colorPos ++ % colors.length];
}

/**
 * Initialize the set of shapes
 * Note: API independent function
 */
function initShapes() {
  shapes = [];
  shapes.push("circle");
  shapes.push("square");
  shapes.push("triangle");
  shapePos = 0;
}

/**
 * get next available shape
 */
function getNextShape() {
	return shapes[shapePos ++ % shapes.length];
}

/**
 * get next available style
 */
function getStyle(type) {
	
	if(!conceptStyles[type]) {
		conceptStyles[type] = {"$color" : getNextColor() , "$type" : getNextShape()};
	}
	return conceptStyles[type];
}

function openEditor(concept,uid) {
  window.open( "/locate?agent-ref=CSCWD-ONTO&" + getLanguage(window.location.href) + "&class-ref=" + concept + "&caller=index" +
      "&id-ref="+uid, '_blank' );
}

/* EOF */