﻿;;;===============================================================================
;;;10/05/29
;;;                               AGENT CIT-MOTHER
;;;
;;;===============================================================================
#|
For example, let an OMAS agent named Mother be working in a SUGA coterie.
When a message (create Child-1) is sent from <suga-interface> to the Mother,
the Mother creates an OMAS agent named Child-1 
which is going to write a message Hello-1 onto a console.

Then a message (create Child-2) is sent from <suga-interface> to the Mother,
the Mother creates an OMAS agent named Child-2 
which is going to write a message Hello-2 onto a console.

This is a start point to develop a self-organization multi-agent system.

I named the agent CIT-MOTHER to avoid possible conflicts with a person 
ontology.
When the agent CIT-MOTHER receives a :CREATE inform message, it creates
a new CHILD agent with an :HELLO method that prints the hello message 
into the Lisp console after it receives the HELLO message.

Note that a CHILD agent must be created in its own package, which means
that we must create its package first...

2010
 0529 OMAS v8 requires that each agent be created in its own package
|#


(defpackage :CIT-MOTHER (:use :cl #+MCL :ccl :moss :omas))
(in-package :CIT-MOTHER)

(omas::defagent :CIT-MOTHER :redefine t)

;;;===============================================================================
;;; 
;;;                         service macros and functions 
;;;
;;;===============================================================================


;;; ==================================== globals =================================
;;; Globals  can be used within the agent space to simplify programming, although
;;; it is better to use the agent memory.

(defparameter *child-counter* 0 "counter for naming children")

;;;===============================================================================
;;; 
;;;                                    skills 
;;;
;;;===============================================================================

;;; ================================= skill section ======================= CREATE

(defParameter *CIT-MOTHER-CREATE-functions*
  '((:functions static-CREATE 
                dynamic-CREATE)))
				
(defun static-hi (agent message)
   "skill for CHILD-nn agent to print hello message"
   ;; print message into the Lisp console
   (format t "~%===> Hi from ~S" (omas::key agent))
   (static-exit agent nil))
	
(defskill :CREATE :CIT-MOTHER
  :static-fcn static-CREATE)

(defun static-CREATE (agent environment)
  "create a new agent and sends a message to it"
  (let ((child-key (intern (format nil "CHILD-~S" (incf *child-counter*)) :keyword))
        ask-message)
  ;; create a new agent
  ;; first we must create new agent package JPB 1005
  (make-package child-key :use '(:cl #+MCL :ccl :moss :omas))
  (with-package child-key
    (make-agent child-key)
    ;; give it a skill
    (make-skill :hi child-key :static-fcn 'static-hi)
    )
  ;; create message
  (setq ask-message 
     (make-instance 'omas::message
	    :type :request
		:from (omas::key agent)
		:to child-key
		:action :hello))
  ;; send message after half a second
  (sleep .5)
  (send-message ask-message)
  ;;exit
  (static-exit agent nil)))
  
:EOF