﻿;;;-*- Mode: Lisp; Package: "TYLER" -*-
;;;===============================================================================
;;;10/05/19
;;;             AGENT TYLER: ONTOLOGY (file tyler-ontology.lisp)
;;;             Copyright Barthès@UTC, 2009, 2010
;;;
;;;=============================================================================== 

#|
TYLER is the assistant agent of KS.
It can find email addressses, telephone numbers, or use web services (?)

Its ontology is used to process dialogs, answer requests or trigger actions, i.e.
send messages to other agents.
The current ontology contains information about addresses, publications and 
research specialties.
However, Tyler is not a specialist of those subjects and its ontology is rather
shallow. It contains enough vocabulary to ask a specialist (one of the staff agent).

Questions asked to TYLER:
   - what is the telephone number of "Trigano"?
   - phone "dbb"?
   - home phone of "Moulin"?
   - extension of DICIT?
   - send message to "gapenne".
   - ...
   
2010
 0519 upgrading to version 8.0   
|#

(eval-when (compile load eval)
  (unless (find-package :tyler) (make-package :tyler)))

(in-package :tyler)

(eval-when (compile load eval)
  (use-package :moss)
  (use-package :omas)
  ) ; reexport it for use in other packages 

(defontology 
  (:title "Tyler ontology")
  (:version "1.0")
  (:language :en)
  )

(format t "~%;*** Loading TYLER ontology, language is: ~S" *language*)

;;;===================================================================== ADDRESSES 

(defchapter 
  (:name :en "addresses"))

;;;--------------------------------------------------------------- GENERAL ADDRESS
(defsection 
  (:name :en "general addresses"))

(defconcept "address"
  (:doc "An ADDRESS can be different things:
   - a postal home address
   - a professional address
   - an email
   - a web address, i.e. the address of a personal web page
Each of those can be multiple."))

;;;---------------------------------------------------------------- POSTAL ADDRESS
(defsection 
  (:name :en "postal addresses"))

(defconcept "postal address; home address"
  (:is-a "address")
  (:doc "A POSTAL ADDRESS od  HOME ADDRESS is the information needed to locate ~
         the place where a person is living.")
  (:att "street and number; street")
  (:att "town; city" (:unique))
  (:att "country" (:unique))
  (:att "zip; postal code" (:unique)))

;;;----------------------------------------------------------------- EMAIL ADDRESS
(defsection
  (:name :en "email addresses"))

(defconcept "email address; email"
  (:is-a "address")
  (:doc "An EMAIL ADDRESS is the code allowing to receive an electronic message.")
  (:att "code"))

;;;------------------------------------------------------------------- WEB ADDRESS
(defsection
  (:name :en "web addresses"))

(defconcept "web address; web page; personal web page"
  (:doc "A WEB ADDRESS is the URL of a web page containing personal data.")
  (:att "URL"))

;;;======================================================================== PEOPLE 

(defchapter
  (:name :en "people"))

(defconcept "person"
  (:att "name" (:entry))
  (:att "first-name" (:entry))
  (:rel "home address" (:to "home address"))
  (:rel "email" (:to "email"))
  (:rel "web page" (:to "web address"))
  (:rel "huspand" (:to "person")(:unique))
  (:rel "wife" (:to "person")(:unique))
  (:rel "mother" (:to "person"))
  (:rel "personal assistant" (:to "omas agent"))
  )

(defconcept "student"
  (:is-a "person")
  (:rel "school" (:to "university"))
  )

;;;============================================================ PUBLISHED MATERIAL

(defchapter
  (:name :en "published material"))

;;;================================================================= ORGANIZATIONS 

(defchapter
  (:name :en "organizations"))

(defconcept "organization"
  (:att "name" (:entry))
  (:att "acronym" (:entry))
  (:rel "address" (:to "address"))
  (:rel "phone" (:to "phone"))
  (:rel "web page" (:to "web page"))
  (:doc "An ORGANIZATION is a social structure private or public that has some ~
         social goals.")
  )

(defconcept "university"
  (:is-a "organization")
  (:doc "A UNIVERSITY is an organization involved in higher education.")
  )

(defconcept "company; firm; enterprise"
  (:is-a "organization")
  (:doc "A COMPANY, FIRM, or ENTERPRISE usually undertakes activities to make ~
         some profit for its owner(s).")
  )

(defconcept "non profit organization"
  (:is-a "organization")
  (:doc " A NON PROFIT ORGANIZATION is not allowed to derive profits from its ~
         activities.")
  )

;;;====================================================================== TELECOMS 

(defchapter 
  (:name :en "telecoms"))

;;;--------------------------------------------------------------------- TELEPHONE
(defsection 
  (:name :en "telephone"))

(defconcept "phone; telephone"
  (:doc "A TELEPHONE is a means to talk when away. One can have different types ~
         of telephone number: home phone, office phone, cell phone, etc.")
  (:att "number")
  )

(defconcept "home phone; home telephone"
  (:doc "A HOME PHONE is a telephone installed at the private address of the owner.")
  (:is-a "phone")
  )

(defconcept "office phone; office telephone"
  (:doc "An OFFICE PHONE is a telephone installed at the work-place.")
  (:is-a "phone")
  )

(defconcept "cell phone"
  (:doc "A CELL PHONE is a portable telephone.")
  (:is-a "phone")
  )

(definstmethod =summary PHONE ()
  (list (format nil "~{~A~^, ~}" (HAS-NUMBER))))

;;;=============================================================================== 
;;;
;;;                                INDIVIDUALS
;;;
;;;=============================================================================== 

(defindividual "documentation"
  ("title" "global help")
  ("documentation" 
    "This is the global help... As your assistant I can do a certain number of ~
     things for you. They correspond to tasks I know of. The easiest way to ~
     use me is to ask me using simple sentences what you want to do.
     But remember, my vocabulary and understanding is quite limited.")
  )

;;;=============================================================================== 
  
;;; set of OMAS assistants

(defindividual "omas agent"
  ("ag-name" "albert")
  ("ag-key" :albert)
  )
  
(defindividual "omas agent"
  ("ag-name" "alfredo")
  ("ag-key" :alfredo)
  )
  
(defindividual "omas agent"
  ("ag-name" "aristide")
  ("ag-key" :aristide)
  )

  (defindividual "omas agent"
  ("ag-name" "stevens")
  ("ag-key" :stevens)
  )
  
;;; Set of persons  
  
(defindividual "person"
  ("name" "Barthès")
  ("first name" "Jean-Paul")
  ("personal assistant" ("omas agent" ("ag-name" :is "albert")))
  )
  
(defindividual "person"
  ("name" "Gonzales")
  ("first name" "Omar")
  ("personal assistant" ("omas agent" ("ag-name" :is "alfredo")))
  )
  
(defindividual "person"
  ("name" "Ramos")
  ("first name" "Milton")
  ("personal assistant" ("omas agent" ("ag-name" :is "aristide")))
  )
  
(defindividual "person"
  ("name" "Sugawara")
  ("first name" "Kenji")
  ("personal assistant" ("omas agent" ("ag-name" :is "tyler")))
  )
;;;===============================================================================
:EOF