;;;-*- Mode: Lisp; Package: "CG-USER" -*-
;;;===============================================================================
;;;11/02/15
;;;        U T C <-> N O T E B O O K - T E S T S (udp-tests.lisp)
;;;
;;; Low level tests for checking remote UDP connections using unicode
;;;===============================================================================

#|
2011
 0215 creation
|#

(in-package :cg-user)

(eval-when (:compile-toplevel :load-toplevel :execute)
  (require :sock))

(defparameter *site* :UTC)
(defparameter *site-ip* "172.17.130.153" "UTC internal")

(defparameter *site-port* 52011)

(defparameter *remote* :NOTEBOOK)
(defparameter *remote-ip* "172.26.130.121" "notebook (WIFI) IP")

;;;===============================================================================
;;;
;;;         Declare socket for getting incoming datagrams
;;;
;;;===============================================================================

(defparameter *receive-socket* nil)

;;;===============================================================================
;;;
;;;         Creating process to receive external messages
;;;
;;;===============================================================================

(defvar *receiving-process*)

(defun create-receiving-process ()
  (declare (special *receiving-process* *site-port*))
  (unless *receive-socket*
    (setq *receive-socket*
          (socket:make-socket :type :datagram :local-port *site-port*)))
  (setq *receiving-process*
        (mp:process-run-function 
         (format nil "~S Receiving" *site*) #'receiving *site-port*)))

(defun receiving (&aux message)
  "function called by the process"
  (declare (special *receive-socket*))
  
  (loop
    (multiple-value-bind 
      (raw-buffer size)
        (socket:receive-from *receive-socket* 1024 :extract t)
      (format t "~&receiving/ raw buffer:~& ~S~&  size: ~S" raw-buffer size) 

      ;; transform unicode string into internal string
      ;; truncate is added as a safety measure
      (setq message (excl::octets-to-string raw-buffer :external-format :fat
                                            :truncate t))
      ;; print trace into Lisp console
      (format t "~&===== / receiving message: ~&  ~S" message))
    ))

;;;===============================================================================
;;;
;;;                 Sending messages to remote site
;;;
;;;===============================================================================

(defvar *send-socket*)

(defparameter *count* 0)

(defun create-send-socket ()
  (setq *send-socket*
        (socket:make-socket :remote-host *remote-ip* :remote-port 52011)))

(defun send-new-message ()
  "create a new message and send a datagram"
  (declare (special *count* *site* *send-socket* *remote-ip*))
  (let ((message (format nil "Test message from ~S #~S." *site* (incf *count*)))
        test errno buffer)
    
    ;; convert fragment to a byte array using unicode :fat format (big endian)
    (setq buffer (excl::string-to-octets message :external-format :fat))
    (format t "~&*** Message to be sent:~&  ~S" buffer)

    (multiple-value-setq (test errno)
      (ignore-errors
       (socket:send-to *send-socket* buffer (length buffer)
                       :remote-host *remote-ip* :remote-port 52011))
      )
    
    (if (and (null test) errno)
        ;; could not send message for some reason
        (progn
          (cg:beep)
          (format t "~&*** Can't send message to: ~S. ~&  Error: ~S~&Message: ~S" 
            *remote-ip* errno buffer))
      ;; otherwise OK
      (format t "~&*** message was sent."))
    ))

;;;===============================================================================
;;;
;;;         Killing process and closing socket for reset
;;;
;;;===============================================================================

(defun reset-everything ()
  (when *receive-socket* (close *receive-socket*)
    (setq *receive-socket* nil))
  (when *receiving-process* (mp:process-kill *receiving-process*)
    (setq *receiving-process* nil))
  (when *send-socket* (close *send-socket*)
    (setq *send-socket* nil))
  )

;;;===============================================================================
;;;
;;;         Tests
;;;
;;;===============================================================================

#|

;;; create process and allow clients to connect on the socket
(create-receiving-process) 

;;; can check processes that a new process has been created and is waiting
;;; process name is ":NOTEBOOK-UTC Receiving"

;;;========== SENDING

;;; Try to send a message: 
;;;     "Test message from <local-site> #nn."

(send-new-message)

;;; The receiving function on the other site should print into the Lisp console:
;;;
;;; "=====
;;;  NOTEBOOK-UTC/ receiving/ incoming message: 
;;;       Test message from <remote-site> #nn.

;;; If everyting is OK, on can send as many messages as one wants

;;; If something goes wrong the following function resets everything

(reset-everything)

|#
:EOF