﻿;;;-*- Mode: Lisp; Package: "NEWS" -*-
;;;===============================================================================
;;;10/12/07
;;;             AGENT NEWS: ONTOLOGY (file NEWS-ontology.lisp)
;;;             Copyright Barthes@UTC, 2007-2010
;;;
;;;=============================================================================== 

;;;********** WARNING
;;; The HAS-XXX macros cannot be used in the definitions of methods when XXX is a 
;;; property defined in MOSS.
;;;**********

#|
NEWS is a staff agent that handles a file of news.Its ontology describes news
and categories, keywords, temporal concepts (year month).

Questions asked to OMAR:
   - publier une nouvelle.
   - quelque chose s'est passée
   - quoi de neuf?
   - dérnieres nouvelles?
   - montrer nouvelles sur economy
   - ...

2007
 1101 Creation
 1128 adding a field baddress to publications
      changing éditeur to coordonnateur for translating Editor of a book
2008
 0604 removing use-package
 1231 adding macros for easier printing
2009
 0125 converting to UTF-8
2010
 1206 adding test news items
|#

(in-package :NEWS)

;;;===============================================================================
;;;
;;;                              Macros
;;; 
;;;===============================================================================
#|
;;; what on earth are those macros for?

(defMacro insert-if (property)
  `(or (if (,property) (format nil "~A, " (car (,property))) "")))

;;; list of summaries of neighbors

(defMacro insert-list-if (property)
  `(or (if (,property) (format nil "~{~A~^, ~}A, "
                               (reduce #'append 
                                       (broadcast (,property) '=summary))) "")))

? (insert-if has-baddress)
;; expands to
(IF (HAS-BADDRESS) (FORMAT NIL "~A, " (CAR (HAS-BADDRESS))) "")

? (insert-list-if has-editor)
;; expands to
(if (HAS-EDITOR)
  (format NIL
          "~{~A~^, ~}A, "
          (reduce #'APPEND (BROADCAST (HAS-EDITOR) '=SUMMARY)))
  "")
|#
;;;===============================================================================
;;;
;;;                              Ontology
;;; 
;;;===============================================================================

(defontology 
  (:title "NEWS ontology")
  (:version "1.0")
  (:language :all)
  )

;;;-------------------------------------------------------------------------------
;;;                                NEWS
;;;-------------------------------------------------------------------------------

(defconcept (:en "news" :fr "nouvelle" :es "noticia")
  (:att (:en "title" :fr "sujet" :es "Título") (:entry))
  (:att (:en "content" :fr "contenu" :es "Contenido"))
  (:att (:en "date" :fr "date" :es "fecha"))
  (:att (:en "hour" :fr "heure"  :es "hora"))  
  (:att (:en "author" :fr "auteur"))  
  (:rel (:en "section" :fr "section" :es "sección")(:to "section"))
  )

;;;-------------------------------------------------------------------------------
;;;                              SECTION
;;;-------------------------------------------------------------------------------

(defconcept (:en "section" :fr "section" :es "sección")
  (:doc :en "A SECTION is the category of a new."
        :fr "La SECTION désigne la catégorie d'une nouvelle."
        :es "Una sección es una categoría de una noticia")
  (:att (:en "name" :fr "nom" :es "nombre")(:entry)))


;;;===============================================================================
;;;
;;;                              Methods
;;; 
;;;===============================================================================

;;;---------------------------------------------------------------- NEWS: =summary

(definstmethod =summary news ()
  (list (format nil "~A ~A (~A) : ~A~%    ~A" 
                (car (has-date))
	        (car (has-hour))
	        (or (car (has-author)) "anonyme")
	        (or (car (has-title)) "no title")
                (car (has-content)))))

#|
? (send '$E-NEWS.1 '=summary)
("6-dec-2010 18:00 (JP Barthès) : bulletin météo
    Il neige actuellement à Compiègne...")
|#
;;;------------------------------------------------------------- SECTION: =summary

(definstmethod =summary section () 
  (list (mln::get-canonical-name (car (has-name)))))

#|
? (send '$E-SECTION.2 '=summary)
("Économie")
|#

;;;===============================================================================
;;;===============================================================================
;;;
;;;                                INDIVIDUALS
;;;
;;;===============================================================================
;;;===============================================================================


;;;========================================================================== NEWS 

;;; Some news for testing

(defindividual "news"
  ("date" "6-dec-2010")
  ("hour" "18:00")
  ("author" "JP Barthès")
  ("title" "bulletin météo")
  ("content" "Il neige actuellement à Compiègne...")
  ("section" "Weather"))

(defindividual "news"
  ("date" "6-dec-2010")
  ("hour" "12:00")
  ("author" "JP Barthès")
  ("title" "bulletin sportif")
  ("content" "La Bulgarie vient de gagner la coupe Davis.")
  ("section" "Sports"))

(defindividual "news"
  ("date" "6-dec-2010")
  ("hour" "19:00")
  ("author" "JP Barthès")
  ("title" "nouveau bulletin météo")
  ("content" "La température est positive, la neige ne devrait pas tenir.")
  ("section" "Weather"))

;;;====================================================================== SECTIONS 

(defindividual "section"
  ("name" (:en "Culture" :fr "Culture" :es "Cultura"))
  )

(defindividual "section"
  ("name" (:en "Economy" :fr "Économie" :es "Economía"))
  )

(defindividual "section"
  ("name" (:en "International" :fr "International" :es "Internacional"))
  )

(defindividual "section"
  ("name" (:en "National" :fr "National" :es "Nacional"))
  )

(defindividual "section"
  ("name" (:en "OMAS-MOSS"))
  (:doc :en "OMAS-MOSS section gathers news about OMAS-MOSS")
  )

(defindividual "section"
  ("name" (:en "Other" :fr "Divers"))
  )

(defindividual "section"
  ("name" (:en "Society" :fr "Société" :es "Sociales"))
  )

(defindividual "section"
  ("name" (:en "Sports" :fr "Sport" :es "Deportes"))
  )

(defindividual "section"
  ("name" (:en "TWA"))
  (:doc :en "TWA denotes the Trans World Agent project")
  )

(defindividual "section"
  ("name" (:en "Weather" :fr "Météo" :es "Clima"))
  )

;;;===============================================================================

:EOF