﻿;;;-*- Mode: Lisp; Package: "PATIENT" -*-
;;;===============================================================================
;;;11/04/22
;;;                                AGENT PATIENT 
;;;
;;;===============================================================================

#|
The PA for the patient of the CAAP project.
We test the mechanism of special interaction window.

2011
 0414 Creation 
|#

(defpackage :PATIENT (:use :cl #+MCL :ccl :moss :omas))
(in-package :PATIENT)

;;;===============================================================================
;;;                                  Agent
;;;===============================================================================

(omas::defagent :PATIENT :language :EN)

;;;===============================================================================
;;;                                 GLOBALS
;;;===============================================================================

(defparameter *interface* nil)

(defparameter *PATIENT-home* 
   (make-pathname
        :host (pathname-host *load-pathname*)
        :device (pathname-device *load-pathname*)
        :directory (butlast (pathname-directory *load-pathname*))
		)
	"path to folder containing this code")

(defparameter *default-patient-timeout* 10)
		 
; (defparameter *topic-list* 
    ; '("Entertainment" "Sports" "Family & Life" "Media & Hobby"
	  ; "Politics & Economy" "Natural Science" "Health & Medical"
	  ; "Materials"))
	  
;; list of empty topics at the beginning of the doalog
(defparameter *topic-list* 
    '("" "" "" "" "" "" "" ""))
;;;========================================================================= skill
;;;                                 ASK
;;;===============================================================================

(defskill :ask :PATIENT
  :static-fcn static-ask
  )

(defun static-ask (agent message args)
  "receive a message from therapist and post it. We wait for an ansnwer ~
   from the PATIENT during 10s.
Arguments:
   agent: agent
   message: incoming message (ignored)
   text: a string
Return:
   :done"
  (declare (ignore message))
  (let ((text (cdr (assoc :data args)))
        ;; check if the message specified a timeout
        (timeout (cdr (assoc :timeout args)))
		(interface (getw agent :patient-window))
		)
    (format t "~%; patient ask #1/*interface*: ~S" interface)
	;; if the PATIENT window is not there, create it
	(unless interface
	    (let ((msg (make-instance 'omas::message  :type :display 
		                    :action :create
                            :from :PATIENT :to :PATIENT
                            :args '(:patient-window PATIENT-WINDOW 
							        make-PATIENT-window))))
          ;; create window
          (send-message msg)
		  ;; wait a litte bit until the window is created
		  (sleep 0.5) ; 500ms
	      ;; the window id is recovered by (getw agent :patient-window)
		  (setq interface (getw agent :patient-window))
		  ))
    (format t "~%; patient ask #2/*interface*: ~S" interface)
	;; clear answer pane
	(setf (cg:value (cg:find-component :input-pane interface)) "")
    ;; post message
    (setf (cg:value (cg:find-component :answer-pane interface)) text)
    ;; wait for an answer during 10s. The answer is posted in the answer slot of the
    ;; agent
	(if
     (mp:process-wait-with-timeout "Waiting for input" 
	                               (or timeout *default-patient-timeout*) 
								   #'recall agent :answer)
	 ;; if t, we got an answer
	 (let ((answer (recall agent :answer)))
        (format t "~%; patient ask #3/answer: ~S" (recall agent :answer))
		;; reset trigger
		(remember agent nil :answer)
		;; return answer
	    (static-exit agent answer)
		)
     ;; if nil, we have a timeout (the patient did not answer)
     (static-exit agent :failure))))

#|
(static-ask sa_PATIENT nil 
         '((:data . "Did you think I would forget the palace?")))
|#
;;;========================================================================= skill
;;;                             POST TOPICS
;;;===============================================================================

(defskill :post-topics :PATIENT
  :static-fcn static-post-topics
  )

(defun static-post-topics (agent message args)
  "Post a list of topics as hotspots and ask question.
Arguments:
   agent: agent
   message: incoming request
   args: something like ((:data (<topic list>) <question>))
Return:
   the topic corresponding to the hotspot or failure."
  (declare (ignore message))
  (let* ((data (cdr (assoc :data args)))
         (topic-list (car data))
		 (question (cadr data))
		 ;; patient interface
		 (interface (getw agent :patient-window)) 
		 ;; doctor area
		 (question-pane (cg:find-component :answer-pane interface))
		 answer)
	; (format t "~%; static-post-topics / ~
	                        ; ~% data: ~S ~
							; ~% topic-list: ~S ~
							; ~% question: ~S ~
							; ~% interface: ~S ~
							; ~% question-pane: ~S"
		; data topic-list question interface question-pane)
	;; post topics
	(post-topics agent interface topic-list)
	(cg:invalidate interface)
    ;; post the question
    (setf (cg:value question-pane) question)
	;; wait before suspending process
	(sleep 0.2)
    ;; wait with timeout
    ;; if it returns t, then we got a click from user, otherwise we have a timeout
    (if
     (mp:process-wait-with-timeout "Waiting for input" 
	                               *default-patient-timeout* 
								   #'recall agent :answer)
	 ;; if t, the user clicked a topic
	 (progn 
	    (setq answer (recall agent :answer))
        (format t "~%; patient post-topics #3/answer: ~S" (recall agent :answer))
		;; reset trigger
		(remember agent nil :answer)
		)
     ;; if nil, we have a timeout (the patient did not answer)
	 (setq answer :failure))
	 
    ;; reset topics?	 
    ;; return answer
    (static-exit agent answer)))
  
#|
(static-post-topics sa_PATIENT nil '((:DATA ("palace" "Nijo castle" "Kiyomizu")
   "Have you ever been to any place listed here?")))
|#
;;;========================================================================= skill
;;;                                 RESET
;;;===============================================================================

(defskill :reset :PATIENT
  :static-fcn static-reset
  )

(defun static-reset (agent message)
  "reset the PATIENT PA agent"
  (declare (ignore message))
  ;; reset the list of objects to sell

  (static-exit agent :done))
  
#|
(static-reset sa_PATIENT nil)
|#
;;;========================================================================= skill
;;;                                  START
;;;===============================================================================

(defskill :start :PATIENT
  :static-fcn static-start
  )

(defun static-start (agent message)
  "initialize the bidding, creating a window"
  (declare (ignore message))
  ;; create a message for opening the window
  ;; doing this makes the scan process create the window so that it does not
  ;; disappear right away (since the scan process is active when the agent is alive)
  (let ((msg (make-instance 'omas::message  :type :display :action :create
                            :from :PATIENT :to :PATIENT
                            :args '(:patient-window PATIENT-WINDOW 
							           create-PATIENT-window))))
    ;; create window
    (send-message msg)
	;; recover the window id (for debugging purposes)
	;; the window is recorded in  (omas::displays agent)
	(setq *interface* (getw agent :patient-window))
    ;; exit
    (static-exit agent :done)))
  
#|
(static-start sa_PATIENT nil)
|# 
;;;========================================================================= skill
;;;                                 TELL
;;;===============================================================================

(defskill :tell :PATIENT
  :static-fcn static-tell
  )

(defun static-tell (agent message args)
  "receive a message from therapist and post it. We wait 2s.
Arguments:
   agent: agent
   message: incoming message (ignored)
   text: a string
Return:
   :done"
  (declare (ignore message))
  (let ((text (cdr (assoc :data args)))
        ;; check if the message specified a timeout
        (timeout (cdr (assoc :timeout args)))
		(interface (getw agent :patient-window))
		)
    (format t "~%; patient tell #1/*interface*: ~S" interface)
	;; if the PATIENT window is not there, create it
	(unless interface
	    (let ((msg (make-instance 'omas::message  :type :display 
		                    :action :create
                            :from :PATIENT :to :PATIENT
                            :args '(:patient-window PATIENT-WINDOW 
							        make-PATIENT-window))))
          ;; create window
          (send-message msg)
		  ;; wait a litte bit until the window is created
		  (sleep 0.5) ; 500ms
	      ;; the window id is recovered by (getw agent :patient-window)
		  (setq interface (getw agent :patient-window))
		  ))
    (format t "~%; patient tell #2/*interface*: ~S" interface)
	;; clear answer pane
	(setf (cg:value (cg:find-component :input-pane interface)) "")
    ;; post message
    (setf (cg:value (cg:find-component :answer-pane interface)) text)
    (static-exit agent :failure)))

#|
(static-tell sa_PATIENT nil 
         '((:data . "Did you think I would forget the palace?")))
|#
;;;===============================================================================
;;;
;;;                               Initial state
;;;
;;;===============================================================================


;(deffact SA_PATIENT 1800 :money)

;;;===============================================================================
;;;===============================================================================
;;;                              Patient window
;;;===============================================================================
;;;===============================================================================

;;; <                         table width                          >
;;; +--------------------------------------+-----------------------+  -
;;; +--------------------------------------+-----------------------+  -
;;; |    |   |   |   Button Area                                   | header height
;;; +--------------------------------------+-----------------------+  -
;;; |                   |                  |                       |
;;; |        Topic table                   |     Avatar Area       | topic height
;;; |                                      |                       |
;;; |<        topic width                 >|                       |
;;; |                   |                  |                       |   table height
;;; +--------------------------------------+-----------------------+ -
;;; |                       Therapist Answers                      | answer  height
;;; |                                                              |
;;; +--------------------------------------+-----------------------+ -
;;; |                       Patient Input Pane                     | input height
;;; |                                                              |
;;; +--------------------------------------+-----------------------+ -
;;; +--------------------------------------+-----------------------+ -
;;; 
(defparameter *nb-of-columns* 2)
(defparameter *nb-of-rows* 4)
(defparameter *max-nb-of-topics* 8)
(defparameter *header-height* 40)
(defparameter *margin* 5)
(defparameter *cell-width* 200)
(defparameter *cell-height* 50)
(defparameter *topic-width* (+ (* (1+ *nb-of-columns*) *margin*)
                               (* *cell-width* *nb-of-columns*)))
(defparameter *topic-height* (+ (* (1+ *nb-of-rows*) *margin*)
                                (* *cell-height* *nb-of-rows*)))
(defparameter *avatar-width* 200)
(defparameter *table-width* (+ *topic-width* *avatar-width*))
(defparameter *voffset* 10)
(defparameter *hoffset* 5)
(defparameter *answer-height* 100)
(defparameter *input-height* 100)
(defparameter *table-height* (+  
                                *header-height*
                                *topic-height*
								*answer-height*
								*input-height*
								))

;;;=================================================================== compute-box

(defun compute-box (nn)
  "computes the location of the nth list element"
  (let ((ii (floor (/ nn *nb-of-columns*))) 
        (jj (rem nn *nb-of-columns*)))
    (cg:make-box 
	   (+ (* jj *cell-width*) (* (1+ jj) *margin*))
	   (+ 0 (* ii *cell-height*) (* (1+ ii) *margin*))
       (* (1+ jj) (+ *margin* *cell-width*))
	   (+ 0 (* (1+ II) (+ *margin* *cell-height*))))))
  
;;;================================================================================
;;;                              CLASSES
;;;================================================================================
   
(defclass patient-window (cg:dialog omas::omas-window)())
   
;;;===============================================================================
;;;                              METHODS
;;;===============================================================================

;;;===============================================================================
;;;                              MAIN WINDOW
;;;===============================================================================

;;;--------------------------------------------------------- CREATE-PATIENT-WINDOW

(defun create-patient-window (window-name window-class)
  "funtion called by OMAS to create agent window.
Arguments:
   window-name: name of the window, usually a keyword
   window-class: the class of the window, here PATIENT-WINDOW
Return:
   the window object."
   (make-patient-window :name window-name :class window-class))
   
;;;----------------------------------------------------------- MAKE-PATIENT-WINDOW
|#
(make-patient-window :topic-list '("palace" "kyoto"))
|#
		  
(defun make-patient-window (&key (left 200)(top 200)
                              (width *table-width*)(height *table-height*) 
							  (topic-list *topic-list*)
							  (name :patient-window)
							  (class 'patient-window)
							  &allow-other-keys)
  (declare (ignore lrest))
  (format t "~%; make-patient-window / process ~S" mp:*current-process*)
  (let* ((frame (cg:make-window name
                  :class class 
                  :owner (cg:screen cg:*system*)
                  :title "TOPICS FOR DISCUSSION"
				  :background-color cg::yellow
                  :exterior (cg:make-box-relative left top
				                    width (+ 18 height))
				  ;:dialog-items (make-patient-dialog-items)
                  :resizable nil
				  :maximize-button nil
                  :scrollbars nil
                  :state :shrunk))
		topic-label box)
    ;; try to add topic area
 	(dotimes (kk *max-nb-of-topics*)
	   (setq box (compute-box kk))
	   (setq topic-label 
	      (make-instance 'cg:scrolling-static-text
		     :name (intern (format nil "TOPIC-~S" kk) :keyword)
			 :value (nth kk topic-list)
             :background-color (cg:make-rgb :red 200 :green 200 :blue 255)
             :left (cg:left box) :top (cg:top box) 
			 :width (cg:width box) :height (cg:height box)
			 :on-click 'select-topic-on-click
			 :on-mouse-in 'change-color
			 :on-mouse-out 'restore-color
             ))
	   (cg:add-component topic-label frame))
	;; context area, where we post the topic of the discussion
	(cg:add-component
	   (make-instance 'cg:static-text
	         :name :current-topic
             :background-color (cg:make-rgb :red 200 :green 200 :blue 255)
			 :left *hoffset*
			 :top (+ *topic-height* *margin*)
			 :height *header-height*
			 :width (- *table-width* (* 2 *hoffset*) 6)
			 :value "<topic being discussed: none yet>"
			 )
		frame)
	;;=== avatar area
	;; nothing there yet
	;;=== answer area
	(cg:add-component
	   (omas::cr-text-item 
	      :left *hoffset* 
		  :top (+ *header-height* *topic-height* *voffset*)
		  :width (- *table-width* (* 2 *hoffset*) 6)
		  :height (- *answer-height* (* 2 *voffset*))
		  :name :answer-pane)
	   frame)
	;;=== input area
	(cg:add-component
	   (omas::cr-text-item 
	      :left *hoffset* 
		  :top (+ *header-height* *topic-height* *answer-height*)
		  :width (- *table-width* (* 2 *hoffset*) 6) 
		  :height (- *answer-height* (* 2 *voffset*))
		  #-MCL :on-change #-MCL input-on-change
		  :name :input-pane)
	   frame)	
	(cg:select-window frame)
	frame))
	
;;;===============================================================================
;;;                           INTERACTION FUNCTIONS
;;;===============================================================================

;;;------------------------------------------------------------------ CHANGE-COLOR

(defun change-color (item button-state last-window)
  "when the mouse enters a topic area, change its color"
   (declare (ignore button-state last-window))
   (unless (or (null (cg:value item))
               (equal (cg:value item) ""))
     (setf (cg:background-color item) 
           (cg:make-rgb :red 252 :green 182 :blue 194)))
   t)

;;;--------------------------------------------------------------- INPUT-ON-CHANGE

(defun input-on-change (item text old-value)
  "Called whenever the content of the input pane (a string) changes.
   We check whether the new char is a period or a question mark 
   if so we terminate input and process it."
  (declare (ignore old-value))
    ;; extract last char and text before the last char, when text is not empty
    (let ((char (if (> (length text) 0) (char text (1- (length text)))))
          (text (if (>= (1- (length text)) 0)
                    (subseq text 0 (1- (length text)))
                  "")))
      ;(format t "~&mw-input-on-change /new char: ~S" char)
      ;; analyse situation:
      ;; - if char is nil do nothing
      ;; - if char is question mark or period, then select the text and send it
      ;;   to THERAPIST
      ;; - otherwise do nothing.
      (cond 
       ;; a null char should not appear unless we erase area
       ((null char)) ; do nothing
       ;;=== question mark
       ((member char (getf omas::*question-markers* *language*))
        ;; select text
        (cg:set-selection item 0 (1+ (length text)))
        ;; add a " ?" at the end of the text
        (remember sa_patient (format nil "~A ~A" text char) :answer))
       ;;=== period
       ((member char (getf omas::*full-stop-markers* *language*))
        ;(format t "~%; mw-input-on-change /char: ~S item: ~S" char item)
        ;; select text
        (omas::%select-all item)
        (remember sa_patient (format nil "~A ~A" text char) :answer))
        )
       )
  ;; return t to accept changes ?
  t)
  
;;;------------------------------------------------------------------- POST-TOPICS
  
(defun post-topics (agent frame topic-list)
   "refreshes the topic area with a new set of topics"
   (let (component)	 
 	(dotimes (kk *max-nb-of-topics*)
	   ;; get component area
	   (setq component 
	         (cg:find-component 
			   (intern (format nil "TOPIC-~S" kk) :keyword)
			   frame))
	   ;; if topic is non nil change the value, otherwise blank area
	   (setf (cg:value component) (or (nth kk topic-list) ""))
       )
	 (cg:set-foreground-window frame)
	 t))
#|
(post-topics sa_patient (cg:find-window :patient-window) 
     '("palace" "Nijo castle" "Kyomizu temple"))
|#
;;;----------------------------------------------------------------- RESTORE-COLOR

(defun restore-color (item button-state last-window)
  "when the mouse enters a topic area, change its color"
   (declare (ignore button-state last-window))
   (setf (cg:background-color item) 
         (cg:make-rgb :red 200 :green 200 :blue 255))
   t)
   
;;;--------------------------------------------------------- SELECT-TOPIC-ON-CLICK 

(defun select-topic-on-click (dialog item)
   "Selects one of the topics and show that it is discussed"
   (declare (special sa_patient))
   (let ((topic (cg:value item))
		 )
   ;; post it in the current topic area
   (setf (cg:value (cg:find-component :current-topic dialog))
      (format nil "Topic being discussed is: ~A." topic))
   ;; set the agent memory, to trigger answer
   (remember sa_patient topic :answer)
   t))

;;;===============================================================================

:EOF 