﻿;;;-*- Mode: Lisp; Package: "PATIENT" -*-
;;;===============================================================================
;;;11/06/06
;;;                            PATIENT WINDOW
;;;
;;;===============================================================================

#|
The PA for the patient of the CAAP project.
We test the mechanism of special interaction window.

2011
 0606 Separating the window file from the agent definition file 
|#

(in-package :PATIENT)

;;;===============================================================================
;;;                                 GLOBALS
;;;===============================================================================
		 
(defparameter *topic-list* '("" "" "" "" "" "" "" ""))

;;;===============================================================================
;;;===============================================================================
;;;                              Patient window
;;;===============================================================================
;;;===============================================================================

;;; <                         table width                          >
;;; +--------------------------------------+-----------------------+  -
;;; +--------------------------------------+-----------------------+  -
;;; |    |   |   |   Button Area                                   | header height
;;; +--------------------------------------+-----------------------+  -
;;; |                   |                  |                       |
;;; |        Topic table                   |     Avatar Area       | topic height
;;; |                                      |                       |
;;; |<        topic width                 >|                       |
;;; |                   |                  |                       |   table height
;;; +--------------------------------------+-----------------------+ -
;;; |                       Therapist Answers                      | answer  height
;;; |                                                              |
;;; +--------------------------------------+-----------------------+ -
;;; |                       Patient Input Pane                     | input height
;;; |                                                              |
;;; +--------------------------------------+-----------------------+ -
;;; +--------------------------------------+-----------------------+ -
;;; 
(defparameter *nb-of-columns* 2)
(defparameter *nb-of-rows* 4)
(defparameter *max-nb-of-topics* 8)
(defparameter *header-height* 40)
(defparameter *margin* 5)
(defparameter *cell-width* 200)
(defparameter *cell-height* 50)
(defparameter *topic-width* (+ (* (1+ *nb-of-columns*) *margin*)
                               (* *cell-width* *nb-of-columns*)))
(defparameter *topic-height* (+ (* (1+ *nb-of-rows*) *margin*)
                                (* *cell-height* *nb-of-rows*)))
(defparameter *avatar-width* 200)
(defparameter *table-width* (+ *topic-width* *avatar-width*))
(defparameter *voffset* 10)
(defparameter *hoffset* 5)
(defparameter *answer-height* 100)
(defparameter *input-height* 100)
(defparameter *table-height* (+  
                                *header-height*
                                *topic-height*
				*answer-height*
				*input-height*
				))

;;;=================================================================== compute-box

(defun compute-box (nn)
  "computes the location of the nth list element"
  (let ((ii (floor (/ nn *nb-of-columns*))) 
        (jj (rem nn *nb-of-columns*)))
    (cg:make-box 
	   (+ (* jj *cell-width*) (* (1+ jj) *margin*))
	   (+ 0 (* ii *cell-height*) (* (1+ ii) *margin*))
       (* (1+ jj) (+ *margin* *cell-width*))
	   (+ 0 (* (1+ II) (+ *margin* *cell-height*))))))
  
;;;================================================================================
;;;                              CLASSES
;;;================================================================================
   
(defclass patient-window (omas::omas-assistant-panel)
    ((agent :accessor agent :initarg :agent)
     (answer-to-task-to-do :accessor answer-to-task-to-do :initform nil)
     (pass-every-char :accessor pass-every-char :initform nil)
	 )
	(:documentation "special patient interaction window")
	)
   
;;;===============================================================================
;;;                              METHODS
;;;===============================================================================

;;;----------------------------------------- (PATIENT-WINDOW) MOSS::ACTIVATE-INPUT

(defMethod moss::activate-input ((win PATIENT-WINDOW) &key erase)
  "prints a text into the output pane of the window.
Arguments:
   erase (key): if true erase the content, otherwise select it
Return: unimportant."
  ;; activate the user panel
  ;; the user will enter its data that will be transferred into the to-do
  ;; slot, waking up the resume process
  (cg:select-window win)
  ;; erase pane
  (moss::display-text win "" :erase erase)
  ;; activate the master pane
  (cg:set-focus-component (cg:find-component :input-pane win))
  )

;;;------------------------------------------- (PATIENT-WINDOW) MOSS::DISPLAY-TEXT
;;; we should add to the window a property registering how many lines there are in
;;; the assistant pane (depending on the haight of the font) so that we can scroll
;;; to keep newly printed stuff in view.

(defMethod moss::display-text ((win patient-window) text &key 
                               (erase t) newline
                               final-new-line (header "") clean-pane 
                               (color moss::*black-color*)
                               &allow-other-keys)
  "prints a text into the output pane of the assistant panel.
Arguments:
   text: text to display (must be a simple string)
   clean-pane (key): erase the assistant pane in all cases
   erase (key): if t, erase previous text, otherwise append to it
   final-new-line (key): if t, add newline command at the end of text
   header (key): a string that will be printed in front of the text 
                 (default \"\")
   newline (key): it t, add a new line in front of the header
Return: 
   nil."
  (let* ((output-pane (omas::%find-named-item :answer-pane win))
         )
    (unless (and (stringp text)(stringp header))
      (error "text ~S and header ~S should be strings." text header))
    
    (if final-new-line (setq text (concatenate 'string text "~%")))
    (if newline (setq header (concatenate 'string "~%" header)))
    
    ;; because text or header could contain format directives, we process it
    (setq text (format nil (concatenate 'string header text)))
       
    (when clean-pane
      (omas::%set-value output-pane "")
      (return-from moss::display-text))
    
    (format t "~%; patient moss::display-text /text: ~S" text)
    (if erase
      (omas::%set-value output-pane (format nil text))
      ;; add more text
      (omas::%set-value output-pane 
	       (format nil "~A~A" (omas::%get-value output-pane) text))
      )
    nil))

#|
(moss::display-text (omas::com-window patient::PA_patient) "... encore du texte")
(moss::display-text (com-window albert::PA_albert) "... encore du texte"
                    :header "~%Master> ")
|#
;;;===============================================================================
;;;                              MAIN WINDOW
;;;===============================================================================

;;;--------------------------------------------------------- CREATE-PATIENT-WINDOW

(defun create-patient-window (window-name window-class)
  "funtion called by OMAS to create agent window.
Arguments:
   window-name: name of the window, usually a keyword
   window-class: the class of the window, here PATIENT-WINDOW
Return:
   the window object."
   (make-patient-window :name window-name :class window-class))
   
;;;----------------------------------------------------------- MAKE-PATIENT-WINDOW
|#
(make-patient-window PA_PATIENT :topic-list '("palace" "kyoto"))
|#
		  
(defun make-patient-window (agent &key (left 200)(top 200)
                                 (width *table-width*)(height *table-height*) 
                                 (topic-list *topic-list*)
                                 (name :patient-window)
                                 (class 'patient-window)
                                 &allow-other-keys)
  (declare (ignore lrest agent))
  (format t "~%; make-patient-window / process ~S" mp:*current-process*)
  (let* ((frame (cg:make-window name
                  :class class 
                  :owner (cg:screen cg:*system*)
                  :title "TOPICS FOR DISCUSSION"
                  :background-color cg::yellow
                  :exterior (cg:make-box-relative left top
                                                  width (+ 18 height))
                  ;:dialog-items (make-patient-dialog-items)
                  :resizable nil
                  :maximize-button nil
                  :scrollbars nil
                  :state :shrunk))
         topic-label box)
    ;; try to add topic area
    (dotimes (kk *max-nb-of-topics*)
      (setq box (compute-box kk))
      (setq topic-label 
            (make-instance 'cg:scrolling-static-text
              :name (intern (format nil "TOPIC-~S" kk) :keyword)
              :value (nth kk topic-list)
              :background-color (cg:make-rgb :red 200 :green 200 :blue 255)
              :left (cg:left box) :top (cg:top box) 
              :width (cg:width box) :height (cg:height box)
              :on-click 'select-topic-on-click
              :on-mouse-in 'change-color
              :on-mouse-out 'restore-color
              ))
      (cg:add-component topic-label frame))
    ;; context area, where we post the topic of the discussion
    (cg:add-component
     (make-instance 'cg:static-text
       :name :current-topic
       :background-color (cg:make-rgb :red 200 :green 200 :blue 255)
       :left *hoffset*
       :top (+ *topic-height* *margin*)
       :height *header-height*
       :width (- *table-width* (* 2 *hoffset*) 6)
       :value "<topic being discussed: none yet>"
       )
     frame)
    ;;=== avatar area
    ;; nothing there yet
    ;;=== answer area
    (cg:add-component
     (make-instance 'cg:multi-line-editable-text 
       :left *hoffset* 
       :top (+ *header-height* *topic-height* *voffset*)
       :width (- *table-width* (* 2 *hoffset*) 6)
       :height (- *answer-height* (* 2 *voffset*))
       :name :answer-pane)
     frame)
    ;;=== input area
    (cg:add-component
     (omas::cr-text-item 
      :left *hoffset* 
      :top (+ *header-height* *topic-height* *answer-height*)
      :width (- *table-width* (* 2 *hoffset*) 6) 
      :height (- *answer-height* (* 2 *voffset*))
      #-MCL :on-change #-MCL input-on-change
      :name :input-pane)
     frame)	
    ;; initialize agent slot
    (setf (agent frame) pa_patient)
    (cg:select-window frame)
    frame))
	
;;;===============================================================================
;;;                           INTERACTION FUNCTIONS
;;;===============================================================================

;;;------------------------------------------------------------------ CHANGE-COLOR

(defun change-color (item button-state last-window)
  "when the mouse enters a topic area, change its color"
   (declare (ignore button-state last-window))
   (unless (or (null (cg:value item))
               (equal (cg:value item) ""))
     (setf (cg:background-color item) 
           (cg:make-rgb :red 252 :green 182 :blue 194)))
   t)
   
;;;-------------------------------------------------------------- CLEAN-TOPIC-INFO

(defun clean-topic-info ()
   "erases the area displaying the current topic"
   (let ((win (omas::com-window pa_patient)))
      (when win
	    (setf (cg:value (cg:find-component :current-topic win)) ""))))
		
;;;--------------------------------------------------------------- INPUT-ON-CHANGE
;;; does not seem to work with multiple-line-editable-text

(defUn input-on-change (item text old-value)
  "Called whenever the content of the input pane (a string) changes.
   We check whether the new char is a period or a question mark 
   if so we terminate input and process it."
  (declare (ignore old-value))
    ;;extract last char and text before the last char, when text is not empty
    (let ((char (if (> (length text) 0) (char text (1- (length text)))))
          (text (if (>= (1- (length text)) 0)
                    (subseq text 0 (1- (length text)))
                  "")))
      ; (format t "~&mw-input-on-change /new char: ~S" char)
      ;; analyse situation:
      ;; - if char is nil do nothing
      ;; - if char is question mark selet the text and call process-master-text
      ;;   adding question mark to the text
      ;; - if char is a period, then select the text and call process-master-text
      ;; - if text is a linefeed, beep ??
      ;; - otherwise do nothing.
      (cond 
       ;; a null char should not appear unless we erase area
       ((null char)) ; do nothing
       ;; if the char is a question mark, insert a space
       ; ((char-equal '#\? char)
       ((member char (getf omas::*question-markers* *language*))
        ;; select text
        (cg:set-selection item 0 (1+ (length text)))
        ;; add a " ?" at the end of the text
        (omas::assistant-process-master-text 
         (agent (cg:parent item)) 
         (format nil "~A ~A" text char)))

       ;; when a period, end of the sentence (ACL does not know #\Enter!)
       ((member char (getf omas::*full-stop-markers* *language*))
        (format t "~%; mw-input-on-change /char: ~S item: ~S" char item)
        ;; select text
        ; (cg:set-selection item 0 (1+ (length text)))
        (omas::%select-all item)
        (omas::assistant-process-master-text (agent (cg:parent item)) text)
        )
       ;; why should we cg:beep on linefeed (because it is not an enter command?)
       ((char-equal '#\Linefeed char)
        (cg:beep))
       ))
  ;; return t to accept changes ?
  t)
  
;;;------------------------------------------------------------------- POST-TOPICS
  
(defun post-topics (agent frame topic-list)
   "refreshes the topic area with a new set of topics"
   (let (component)	 
 	(dotimes (kk *max-nb-of-topics*)
	   ;; get component area
	   (setq component 
	         (cg:find-component 
			   (intern (format nil "TOPIC-~S" kk) :keyword)
			   frame))
	   ;; if topic is non nil change the value, otherwise blank area
	   (setf (cg:value component) (or (nth kk topic-list) ""))
       )
	 (cg:set-foreground-window frame)
	 t))
#|
(post-topics Pa_patient (cg:find-window :patient-window) 
     '("palace" "Nijo castle" "Kyomizu temple"))
|#
;;;----------------------------------------------------------------- RESTORE-COLOR

(defun restore-color (item button-state last-window)
  "when the mouse enters a topic area, change its color"
   (declare (ignore button-state last-window))
   (setf (cg:background-color item) 
         (cg:make-rgb :red 200 :green 200 :blue 255))
   t)
   
;;;--------------------------------------------------------- SELECT-TOPIC-ON-CLICK 

(defun select-topic-on-click (dialog item)
   "Selects one of the topics and show that it is discussed"
   (declare (special Pa_patient))
   (let ((topic (cg:value item))
		 (agent (agent dialog))
		 )
   ;; post it in the current topic area
   (setf (cg:value (cg:find-component :current-topic dialog))
      (format nil "Topic being discussed is: ~A." topic))
   ;; store the answer in the to-do slot, which will resume the conversation
   (setf (omas::to-do agent) topic)
   t))

;;;===============================================================================

:EOF 