﻿;;;-*- Mode: Lisp; Package: "PATIENT" -*-
;;;===============================================================================
;;;11/04/23
;;;                                AGENT PATIENT 
;;;
;;;===============================================================================

#|
The PA for the patient of the CAAP project.
We test the mechanism of special interaction window.

2011
 0414 Creation 
|#

(defpackage :PATIENT (:use :cl #+MCL :ccl :moss :omas))
(in-package :PATIENT)

;;;===============================================================================
;;;                                  Agent
;;;===============================================================================

(omas::defassistant :PATIENT :language :EN :interface make-patient-window)

;;;===============================================================================
;;;                                 GLOBALS
;;;===============================================================================

(defparameter *default-patient-timeout* 10)

;;;========================================================================= skill
;;;                                 RESET
;;;===============================================================================
#|
(defskill :reset :PATIENT
  :static-fcn static-reset
  )

(defun static-reset (agent message)
  "reset the PATIENT PA agent"
  (declare (ignore message))
  ;; reset the list of objects to sell

  (static-exit agent :done))
  

(static-reset pa_PATIENT nil)
|#
;;;========================================================================= skill
;;;                                 SAY
;;;===============================================================================
#|
(defskill :say :PATIENT
  :static-fcn static-say
  )

(defun static-say (agent message args)
  "receive a message from therapist and post it. We wait for an ansnwer ~
   from the PATIENT during 10s.
Arguments:
   agent: agent
   message: incoming message (ignored)
   text: a string
Return:
   :done"
  (declare (ignore message))
  (let ((text (cdr (assoc :data args)))
        ;; check if the message specified a timeout
        (timeout (cdr (assoc :timeout args)))
        (interface (getw agent :patient-window))
        )
    (format t "~%; patient say #1/*interface*: ~S" interface)
    ;; if the PATIENT window is not there, create it
    (unless interface
      (let ((msg (make-instance 'omas::message  :type :display 
                   :action :create
                   :from :PATIENT :to :PATIENT
                   :args '(:patient-window PATIENT-WINDOW 
                                           make-PATIENT-window))))
        ;; create window
        (send-message msg)
        ;; wait a litte bit until the window is created
        (sleep 0.5) ; 100ms
        ;; the window id is recovered by (getw agent :patient-window)
        (setq interface (getw agent :patient-window))
        ))
    (format t "~%; patient say #2/*interface*: ~S" interface)
    ;; clear answer pane
    (setf (cg:value (cg:find-component :input-pane interface)) "")
    ;; post message
    (setf (cg:value (cg:find-component :answer-pane interface)) text)
    ;; wait for an answer during 10s. The answer is posted in the answer slot of the
    ;; agent
    (if
        (mp:process-wait-with-timeout "Waiting for input" 
                                      (or timeout *default-patient-timeout*) 
                                      #'recall agent :answer)
        ;; if t, we got an answer
        (let ((answer (recall agent :answer)))
          (format t "~%; patient say #3/answer: ~S" (recall agent :answer))
          ;; reset trigger
          (remember agent nil :answer)
          ;; return answer
          (static-exit agent answer)
          )
      ;; if nil, we have a timeout (the patient did not answer)
      (static-exit agent :failure))))
|#
#|
(static-say pa_PATIENT nil 
         '((:data . "Did you think I would forget the palace?")))
|#
;;;========================================================================= skill
;;;                                  START
;;;===============================================================================
#|
(defskill :start :PATIENT
  :static-fcn static-start
  )

(defun static-start (agent message)
  "initialize the bidding, creating a window"
  (declare (ignore message))
  ;; create a message for opening the window
  ;; doing this makes the scan process create the window so that it does not
  ;; disappear right away (since the scan process is active when the agent is alive)
  (let ((msg (make-instance 'omas::message  :type :display :action :create
               :from :PATIENT :to :PATIENT
               :args '(:topic-window MY-HOTSPOT-FRAME make-PATIENT-window))))
    ;; create window
    (send-message msg)
    ;; recover the window id
    (setq *interface* (getw agent :topic-window))
    ;; exit
    (static-exit agent :done)))
|#
#|
(static-start pa_PATIENT nil)
|# 
;;;===============================================================================
;;;
;;;                               Initial state
;;;
;;;===============================================================================


;(deffact PA_PATIENT 1800 :money)

;;;===============================================================================

:EOF 